package arzumify.polyenergy.api;

import net.minecraft.util.math.Vec3i;

import java.util.ArrayList;

// Provides energy to EnergyReceivers. Can be implemented by anything able to tick and provide energy!
public interface EnergyProvider {
    /**
     * When called by a receiver, attempts to remove energy from the provider.
     * Do not attempt to extract energy until {@link EnergyReceiver#ready(EnergyProvider)} is called by the receiver.
     *
     * @param amount   The target amount of energy to extract
     * @param receiver The receiver that is attempting to extract energy
     * @return The amount of energy actually extracted. If zero, the receiver will not attempt to extract energy until {@link EnergyReceiver#ready(EnergyProvider)} is called.
     */
    long extract(long amount, EnergyReceiver receiver);

    /**
     * When called by the server matchmaker, indicates that this is a possible receiver to call {@link EnergyReceiver#ready(EnergyProvider)} on.
     * This particular overload is for coordinate-based matchmaking.
     *
     * @param receiver         The receiver that exists.
     * @param pointsOfPresence The points of presence of the receiver (can be multiple, e.g. for a network of wires or a large entity).
     */
    void exists(EnergyReceiver receiver, ArrayList<Vec3i> pointsOfPresence);
}