package arzumify.polyenergy.api;

// Receives energy from EnergyProviders. Can be implemented by anything able to tick and receive energy!
public interface EnergyReceiver {
    /**
     * When called by a provider, indicates that the provider exists and is ready to have energy extracted from.
     * Do not attempt to call {@link EnergyProvider#extract(long, EnergyReceiver)} on any provider until this method is called.
     * This method may be called multiple times by the same provider. Please ignore any duplicate calls, as they may occur in the MatchMaking implementation.
     * We recommend using a Set to store the providers that are ready to provide energy.
     *
     * @param provider The provider that exists and is ready to provide energy to this receiver.
     */
    void ready(EnergyProvider provider);

    /**
     * When called by the server matchmaker, indicates that this provider is no longer able to provide energy to this receiver.
     * Wait for another call to {@link EnergyReceiver#ready(EnergyProvider)} before attempting to extract energy from this provider.
     *
     * @param provider The provider that is no longer able to provide energy to this receiver.
     */
    void unready(EnergyProvider provider);
}
