import arzumify.polyenergy.api.EnergyProvider;
import arzumify.polyenergy.api.EnergyReceiver;
import arzumify.polyenergy.impl.CoordinateMatchMaker;
import arzumify.polyenergy.impl.ProviderDetails;
import arzumify.polyenergy.impl.ReceiverDetails;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerBlockEntityEvents;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.math.Vec3i;

import java.util.ArrayList;

// Cables can have ONE provider and MANY receivers.
public class SimpleCodeOnlyCable implements EnergyProvider, EnergyReceiver, ServerBlockEntityEvents.Unload, ServerBlockEntityEvents.Load {
    public final long capacity;
    public final long inputRate;
    public final long outputRate;
    public final String name;
    public final ProviderDetails providerDetails;
    public final ReceiverDetails receiverDetails;
    public long energy = 0;
    public EnergyProvider currentProvider;

    public SimpleCodeOnlyCable(Vec3i pos, long capacity, long inputRate, long outputRate, String name) {
        this.capacity = capacity;
        this.inputRate = inputRate;
        this.outputRate = outputRate;
        this.providerDetails = ProviderDetails.NewSimple(pos, this);
        this.receiverDetails = ReceiverDetails.NewSimple(pos, this);
        this.name = name;
        CoordinateMatchMaker.addReceiver(receiverDetails);
    }

    public static void tick(SimpleCodeOnlyCable cable) {
        System.out.println(cable.name + " ticking");
        if (cable.currentProvider != null && cable.energy < cable.capacity) {
            cable.energy += cable.currentProvider.extract(Math.min(cable.capacity - cable.energy, cable.inputRate), cable);
        }
    }

    @Override
    public long extract(long amount, EnergyReceiver receiver) {
        System.out.println(name + " extracting " + amount + " energy");
        long extracted = Math.min(Math.min(outputRate, amount), energy);
        energy -= extracted;
        System.out.println(name + " extracted " + extracted + " energy");
        if (energy == 0) {
            CoordinateMatchMaker.removeProvider(providerDetails);
        }
        return extracted;
    }

    @Override
    public void exists(EnergyReceiver receiver, ArrayList<Vec3i> pointsOfPresence) {
        System.out.println(name + " found other receiver");
        if (currentProvider != null) {
            receiver.ready(currentProvider);
        }
    }

    @Override
    public void ready(EnergyProvider provider) {
        System.out.println(name + " found other provider");
        if (currentProvider == null) {
            currentProvider = provider;
            CoordinateMatchMaker.addProvider(providerDetails);
        }
    }

    @Override
    public void unready(EnergyProvider provider) {
        if (currentProvider == provider) {
            System.out.println(name + " lost other provider");
            currentProvider = null;
            CoordinateMatchMaker.removeProvider(providerDetails);
        }
    }

    @Override
    public void onLoad(BlockEntity blockEntity, ServerWorld serverWorld) {
        CoordinateMatchMaker.addReceiver(receiverDetails);
    }

    @Override
    public void onUnload(BlockEntity blockEntity, ServerWorld serverWorld) {
        CoordinateMatchMaker.removeProvider(providerDetails);
        CoordinateMatchMaker.removeReceiver(receiverDetails);
        currentProvider = null;
    }
}
